import { Injectable } from '@nestjs/common'

import { IAgentServerService } from '@/interface/agent-server.interface'
import { ServiceResponse } from '@/response/response.interface'
import { BusinessCode, HttpStatusCode } from '@/response/status-code.enum'

import { AgentServerHttpService } from './agent-server-http.service'

@Injectable()
export class AgentServerService implements IAgentServerService {
  constructor(private readonly httpService: AgentServerHttpService) {}

  async listStaffs(): Promise<ServiceResponse<StaffsType[]>> {
    const res = await this.httpService.listStaffs()
    if (res.code !== HttpStatusCode.OK) {
      return {
        code: BusinessCode.BAD_REQUEST,
        msg: '获取客服列表失败',
        data: null
      }
    }
    return {
      code: BusinessCode.SUCCESS,
      msg: '获取客服列表成功',
      data: res.data
    }
  }

  async listScenes(): Promise<ServiceResponse<Scene[]>> {
    const res = await this.httpService.listScenes()
    if (res.code !== HttpStatusCode.OK) {
      return {
        code: BusinessCode.BAD_REQUEST,
        msg: '获取场景列表失败',
        data: null
      }
    }
    return {
      code: BusinessCode.SUCCESS,
      msg: '获取场景列表成功',
      data: res.data
    }
  }

  async listModels(): Promise<ServiceResponse<Model[]>> {
    const res = await this.httpService.listModels()
    if (res.code !== HttpStatusCode.OK) {
      return {
        code: BusinessCode.BAD_REQUEST,
        msg: '获取模型列表失败',
        data: null
      }
    }
    return {
      code: BusinessCode.SUCCESS,
      msg: '获取模型列表成功',
      data: res.data
    }
  }

  async getBasePrompt(scene: string): Promise<ServiceResponse<string>> {
    const res = await this.httpService.getBasePrompt(scene)
    if (res.code !== HttpStatusCode.OK) {
      return {
        code: BusinessCode.BAD_REQUEST,
        msg: '获取基础提示词失败',
        data: null
      }
    }
    return {
      code: BusinessCode.SUCCESS,
      msg: '获取基础提示词成功',
      data: res.data
    }
  }

  async getDialogueHistory(
    staffId: string,
    userName: string,
    unionId: string,
    recentMinutes: string
  ): Promise<ServiceResponse<UserInfo[]>> {
    const user = await this.httpServiceListUsers(userName, unionId)
    if (user.length === 0) {
      return {
        code: BusinessCode.BAD_REQUEST,
        msg: '用户不存在',
        data: null
      }
    }

    const dialogueHistory = await Promise.all(
      user.map(async (user) => {
        const res = await this.getDialogueHistoryByUserId(
          staffId,
          user,
          recentMinutes
        )
        return res
      })
    )

    return {
      code: BusinessCode.SUCCESS,
      msg: '获取对话历史成功',
      data: dialogueHistory
    }
  }

  async httpServiceListUsers(
    userName: string,
    unionId: string
  ): Promise<UserProfile[]> {
    const res = await this.httpService.listUsers(userName, unionId)
    if (res.code !== HttpStatusCode.OK) {
      return []
    }
    return res.data
  }

  async getDialogueHistoryByUserId(
    staffId: string,
    user: UserProfile,
    recentMinutes: string
  ): Promise<UserInfo> {
    const res = await this.httpService.getDialogueHistory(
      staffId,
      user.third_party_user_id,
      recentMinutes
    )

    const userInfo: UserInfo = {
      ...user,
      history: res.data || []
    }

    return userInfo
  }
}
